/*
 * RandPass.java
 *
 * Created on 6 de agosto de 2007, 1:02
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package kemit.crypto;

/**
 *
 * @author JL
 */
    
    /** Creates a new instance of RandPass */
import java.security.SecureRandom;
//import gnu.getopt.*;
import java.text.MessageFormat;
import java.util.ResourceBundle;
import java.util.Locale;
import java.util.Vector;

public class RandPass {

    //protected static ResourceBundle labels = ResourceBundle.getBundle("com.Ostermiller.util.RandPass",  Locale.getDefault());

    private static final int DEFAULT_PASSWORD_LENGTH = 8;

    public static final char[] NUMBERS_AND_LETTERS_ALPHABET = {
        'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3','4','5','6','7','8','9',
    };

    public static final char[] SYMBOLS_ALPHABET = {
        '!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',':',';','<','?','@','[','\\',']','^','_','`','{','|','}','~',
    };

    public static final char[] PRINTABLE_ALPHABET = {
        '!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/','0','1','2','3','4','5','6','7','8',
        '9',':',';','<','?','@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R',
        'S','T','U','V','W','X','Y','Z','[','\\',']','^','_','`','a','b','c','d','e','f','g','h','i','j',
        'k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z','{','|','}','~',
    };

    public static final char[] LOWERCASE_LETTERS_ALPHABET = {
        'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
    };

    public static final char[] LOWERCASE_LETTERS_AND_NUMBERS_ALPHABET = {
        'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p',
        'q','r','s','t','u','v','w','x','y','z','0','1','2','3','4','5','6','7','8','9',
    };

    public static final char[] LETTERS_ALPHABET = {
        'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
    };

    public static final char[] UPPERCASE_LETTERS_ALPHABET = {
        'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
    };

    public static final char[] NONCONFUSING_ALPHABET = {
        'A','B','C','D','E','F','G','H','J','K','M','N','P','Q','R','S','T','W','X','Y','Z','a','b','c',
        'd','e','f','g','h','j','k','m','n','p','q','r','s','t','w','x','y','z','2','3','4','5','6','7','8','9',
    };

    protected SecureRandom rand;

    protected int repetition = -1;

    protected char[] alphabet;
    protected char[] firstAlphabet;
    protected char[] lastAlphabet;

    public RandPass(){
        this(new SecureRandom(), NONCONFUSING_ALPHABET);
    }
    
    public RandPass(SecureRandom rand){
        this(rand, NONCONFUSING_ALPHABET);
    }

    public RandPass(char[] alphabet){
        this(new SecureRandom(), alphabet);
    }

    public RandPass(SecureRandom rand, char[] alphabet){
        this.rand = rand;
        this.alphabet = alphabet;
    }

    private class Requirement {
        public Requirement(char[] alphabet, int num){
            this.alphabet = alphabet;
            this.num = num;
        }
        public char[] alphabet;
        public int num;
    }

    private Vector requirements = null;

    public void addRequirement(char[] alphabet, int num){
        if (requirements == null) requirements = new Vector();
        requirements.add(new Requirement(alphabet, num));
    }

    public void setAlphabet(char[] alphabet){
        if (alphabet == null) throw new NullPointerException("Null alphabet");
        if (alphabet.length == 0) throw new ArrayIndexOutOfBoundsException("No characters in alphabet");
        this.alphabet = alphabet;
    }

    public void setRandomGenerator(SecureRandom rand){
        this.rand = rand;
    }

    public void setFirstAlphabet(char[] alphabet){
        if (alphabet == null || alphabet.length == 0) this.firstAlphabet = null;  else this.firstAlphabet = alphabet;
    }

    public void setLastAlphabet(char[] alphabet){
        if (alphabet == null || alphabet.length == 0) this.lastAlphabet = null;else  this.lastAlphabet = alphabet;            
    }

    public void setMaxRepetition(int rep){ 
        this.repetition = rep - 1;
    }

    public char[] getPassChars(char[] pass){
        boolean verified = false;
        while (!verified){
            int length = pass.length;
            for (int i=0; i<length; i++){
                    char[] useAlph = alphabet;
                    if (i == 0 && firstAlphabet != null){
                            useAlph = firstAlphabet;
                    } else if (i == length - 1 && lastAlphabet != null){
                            useAlph = lastAlphabet;
                    }
                    int size = avoidRepetition(useAlph, pass, i, repetition, useAlph.length);
                    pass[i] = useAlph[rand.nextInt(size)];
            }
            if (requirements != null) applyRequirements(pass);
            verified = true;
            for (int i=0; verified && verifiers != null && i<verifiers.size(); i++){
                    verified = ((PasswordVerifier)verifiers.elementAt(i)).verify(pass);
            }
        }
        return(pass);
    }

    private Vector verifiers = null;

    public void addVerifier(PasswordVerifier verifier){
        if (verifiers == null) verifiers = new Vector();
        verifiers.add(verifier);
    }

    private boolean[] touched = null;
    private int[] available = null;

    private void applyRequirements(char[] pass){
        int size = requirements.size();
        if (size > 0){
            int length = pass.length;
            if (touched == null || touched.length < length) touched = new boolean[length];
            if (available == null || available.length < length) available = new int[length];
            for (int i=0; i<length; i++) touched[i] = false;
            for (int reqNum=0; reqNum<size; reqNum++){
                Requirement req = (Requirement)requirements.elementAt(reqNum);
                int reqUsedInd = req.alphabet.length;
                int fufilledInd = 0;
                int availableInd = 0;
                for (int i=0; i<length; i++){
                    if (arrayContains(req.alphabet, pass[i]) && fufilledInd < req.num){
                        fufilledInd++;
                        touched[i] = true;
                        if (repetition >= 0){reqUsedInd -= moveto(req.alphabet, reqUsedInd, pass[i]);if (reqUsedInd < 0) reqUsedInd = req.alphabet.length;}
                    } else if (!touched[i]){available[availableInd] = i;availableInd++;}
                }

                int toDo = req.num - fufilledInd;
                for (int i=0; i<toDo && availableInd>0; i++){
                    int slot = rand.nextInt(availableInd);
                    char passChar = req.alphabet[rand.nextInt(reqUsedInd)];
                    pass[available[slot]] = passChar;
                    touched[available[slot]] = true;
                    availableInd--;
                    available[slot] = available[availableInd];
                    if (repetition >= 0){ reqUsedInd -= moveto(req.alphabet, reqUsedInd, passChar);if(reqUsedInd < 0) reqUsedInd = req.alphabet.length;}
                }
            }
        }
    }

    private static boolean arrayContains(char[] alph, char c){
        for (int i=0; i<alph.length; i++){
            if (alph[i] == c) return true;
        }
        return false;
    }

    private static int avoidRepetition(char[] alph, char[] pass, int passSize, int repetition, int alphSize){
        if (repetition > -1){
            int repPos = 0;
            while ((repPos = findRep(pass, repPos, passSize, repetition)) != -1){
                alphSize -= moveto(alph, alphSize, pass[repPos+repetition]);
                repPos++;
            }
            if (alphSize == 0) alphSize = alph.length;
        }
        return alphSize;
    }

    private static int findRep(char[] pass, int start, int end, int length){
        for (int i=start; i<end-length; i++){
            boolean onTrack = true;
            for (int j=0; onTrack && j<length; j++){
                if (pass[i+j] != pass[end-length+j]) onTrack = false;
            }
            if(onTrack) return i;
        }
        return -1;
    }

    private static int moveto(char[] alph, int numGood, char c){
        int count = 0;
        for (int i=0; i<numGood; i++){
            if (alph[i] == c){
                numGood--;
                char temp = alph[numGood];
                alph[numGood] = alph[i];
                alph[i] = temp;
                count++;
            }
        }
        return count;
    }

    public char[] getPassChars(int length){
        return(getPassChars(new char[length]));
    }

    public char[] getPassChars(){
        return(getPassChars(DEFAULT_PASSWORD_LENGTH));
    } 

    public String getPass(int length){
        return(new String(getPassChars(new char[length])));
    }

    public String getPass(){
        return(getPass(DEFAULT_PASSWORD_LENGTH));
    }
    
}
