package kemit.ldap.objectos;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.AttributeInUseException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;
import javax.print.AttributeException;
import kemit.ldap.*;


public class GrupoLDAP implements Serializable {
        
    private String cn = "";
    private String gidNumber = "";
    private boolean groupofnames=false;
    private boolean namedobject=false;
    private Hashtable miembros = new Hashtable();
    
    private boolean borrado=false;
    private boolean selected=true;
    
    public GrupoLDAP() {
        
    }        
        
    public UsuarioLDAP getUserbyUid(String uid,Vector usuarios) {
        for (int i=0;i<usuarios.size();i++) {
            UsuarioLDAP u=(UsuarioLDAP) usuarios.get(i);
            if (u.getUid().equals(uid)) return u;
        }
        return null;
    }
    
    public GrupoLDAP(Vector usuarios, SearchResult sr) {        
        Attributes at = sr.getAttributes();
        
        try {
            setCn((String) at.get("cn").get());
            setGidNumber((String) at.get("gidNumber").get());
            
            try {
                
                NamingEnumeration noc = at.get("objectClass").getAll();
                while (noc.hasMore()) {
                    Object o = noc.next();
                    if (o!=null && (o instanceof String)) {
                        String m = (String) o;
                        if (m.equals("namedObject")) setNamedobject(true);
                        if (m.equals("groupOfNames")) setGroupofnames(true);
                    }
                }                
                
                NamingEnumeration ne = at.get("member").getAll();
                while (ne.hasMore()) {
                    Object o = ne.next();
                    if (o!=null && (o instanceof String)) {
                        String m = (String) o;
                        int iuid = m.indexOf("uid=")+4;
                        int fuid = m.indexOf(",",iuid);
                        String uid = m.substring(iuid,fuid);
                        if (!uid.equals("falsouser")) {
                            UsuarioLDAP u=getUserbyUid(uid,usuarios);
                            u.setUid(uid);
                            u.masGrupos(this);
                            masMiembros(uid,u);
                        }
                    }
                }
                for (int i=0;i<usuarios.size();i++) {
                    UsuarioLDAP u=(UsuarioLDAP) usuarios.get(i);
                    if (u.getGidNumber().equals(getGidNumber()))  {
                        u.masGrupos(this);
                        masMiembros(u.getUid(),u);
                    }                    
                }
                
            }  catch (Exception e) {
                
            }                            
        }  catch (NamingException ex) {
            ex.printStackTrace();
        }                
    }

    public boolean newLDAP(DirContext dc,Principal padre) {        
        try {          
            padre.mensageOp("    ---->"+padre.PONE_ATT);
            padre.maxGID++;
            if (getGidNumber().trim().equals("")) setGidNumber(padre.maxGID+"");
            String distinguishedName = "cn=" + getCn() + ",ou=group" ;           
            Attributes newAttributes = new BasicAttributes(true);            
            Attribute oc = new BasicAttribute("objectclass");
            
            oc.add("posixGroup");
          
            oc.add("namedObject");
            oc.add("top");   
            if (padre.getSAMBA()==1) {
                oc.add("sambaGroupMapping");
            }
            newAttributes.put(oc);                        
            
            newAttributes.put(new BasicAttribute("cn", getCn())); 
            newAttributes.put(new BasicAttribute("displayName", getCn()));
            if (padre.getSAMBA()==1) {
                newAttributes.put(new BasicAttribute("sambaGroupType", "2"));
                SambaDomain sd=new SambaDomain(dc);
                newAttributes.put(new BasicAttribute("sambaSID", sd.getSambaSID()+"-"+getGidNumber()));
            }
            newAttributes.put(new BasicAttribute("gidNumber", getGidNumber())); 

            padre.mensageOp("    ---->"+padre.ENVIA_AL_SERVER);
            dc.createSubcontext(distinguishedName, newAttributes);
            padre.mensageOp("    ---->"+padre.CREADO_EN_SERVER);
            return true;
        }
        catch (Exception e) {
            if (padre.debug) padre.mensage(e);
            return false;
        }                
    }
    
    public boolean borrarEnServerLDAP(DirContext dc,Principal padre) {        
        try {     
            dc.destroySubcontext("cn="+getCn()+", ou=group");
            return true;
        } catch (NamingException ex) {
            if (padre.debug) padre.mensage(ex);
            return false;
        }
    }         
    
    public void masMiembros(String uid,UsuarioLDAP u) {
        if (!miembros.contains(u)) miembros.put(uid,u);                        
    }
    
    public void menosMiembros(String uid) {
        miembros.remove(uid);             
    }
    
    public boolean addMemberLDAP(DirContext ctx,UsuarioLDAP u,String DC) throws NamingException {

        if (namedobject) {
            return cambiadeNamedAGroup( ctx,u, DC);
        }
        ModificationItem[] mods = new ModificationItem[1];
 
        Attribute mod0 = new BasicAttribute("member","uid="+u.getUid()+",ou=people,dc="+DC);
        mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE,mod0);

        try {
            ctx.modifyAttributes("cn="+getCn()+",ou=group", mods);       
            return true;
        } catch (AttributeInUseException ae) {
            return true;                          
        } catch (NamingException ex) {
            throw ex;
            //ex.printStackTrace();
            ///return false;
        }        
    }
    
    
    public boolean cambiadeNamedAGroup(DirContext ctx,UsuarioLDAP u,String DC) {
       ModificationItem[] mods = new ModificationItem[1];
  
        try {
            ctx.destroySubcontext("cn="+getCn()+", ou=group");

            String distinguishedName = "cn=" + getCn() + ",ou=group" ;           
            Attributes newAttributes = new BasicAttributes(true);            
            Attribute oc = new BasicAttribute("objectclass");
            
            oc.add("posixGroup");          
            oc.add("groupOfNames");
            oc.add("top");   
            oc.add("sambaGroupMapping");

            newAttributes.put(oc);                        
            newAttributes.put(new BasicAttribute("member", "uid="+u.getUid()+",ou=people,dc="+DC));                        
            newAttributes.put(new BasicAttribute("cn", getCn())); 
            newAttributes.put(new BasicAttribute("displayName", getCn()));
            newAttributes.put(new BasicAttribute("sambaGroupType", "2"));
            SambaDomain sd=new SambaDomain(ctx);
            newAttributes.put(new BasicAttribute("sambaSID", sd.getSambaSID()+"-"+getGidNumber()));            
            newAttributes.put(new BasicAttribute("gidNumber", getGidNumber())); 

            
            ctx.createSubcontext(distinguishedName, newAttributes);            
            setNamedobject(false);
            setGroupofnames(true);
            return true;
            
        } catch (NamingException ex) {            
            ex.printStackTrace();
            return false;
        }                        
    }
    
    public boolean minusMemberLDAP(DirContext ctx,UsuarioLDAP u,String DC) {

        if (miembros.isEmpty())  {
            try {
                ctx.destroySubcontext("cn="+getCn()+", ou=group");
                String distinguishedName = "cn=" + getCn() + ",ou=group" ;           
                Attributes newAttributes = new BasicAttributes(true);            
                Attribute oc = new BasicAttribute("objectclass");

                oc.add("posixGroup");

                oc.add("namedObject");
                oc.add("top");   
                oc.add("sambaGroupMapping");
                newAttributes.put(oc);                        

                newAttributes.put(new BasicAttribute("cn", getCn())); 
                newAttributes.put(new BasicAttribute("displayName", getCn()));
                newAttributes.put(new BasicAttribute("sambaGroupType", "2"));
                SambaDomain sd=new SambaDomain(ctx);
                newAttributes.put(new BasicAttribute("sambaSID", sd.getSambaSID()+"-"+getGidNumber()));
                newAttributes.put(new BasicAttribute("gidNumber", getGidNumber())); 
                ctx.createSubcontext(distinguishedName, newAttributes);
                setNamedobject(true);
                setGroupofnames(false);
                return true;
            } catch (NamingException ex) {
                ex.printStackTrace();
                return false;
            }                        
        } else  {                
            ModificationItem[] mods = new ModificationItem[1];
            Attribute mod0 = new BasicAttribute("member","uid="+u.getUid()+",ou=people,dc="+DC);
            mods[0] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE,mod0);

            try {
                ctx.modifyAttributes("cn="+getCn()+",ou=group", mods);       
                return true;
            } catch (NamingException ex) {            
                //ex.printStackTrace();
                return false;
            }
        }
    }
        
    public String getCn() {
        return cn;
    }

    public void setCn(String cn) {
        this.cn = cn;
    }

    public String getGidNumber() {
        return gidNumber;
    }

    public void setGidNumber(String gidNumber) {
        this.gidNumber = gidNumber;
    }

    public Hashtable getMiembros() {
        return miembros;
    }

    public void setMiembros(Hashtable miembros) {
        this.miembros = miembros;
    }
    
    

    public String toString() {
        return "cn="+getCn()+",gid="+getGidNumber();
    }

    public boolean equals(Object o) {
        if (o==null) return false;
        if (o instanceof GrupoLDAP) {
            GrupoLDAP u=(GrupoLDAP)o;            
            return getCn().equals(u.getCn()) ;            
        } else return false;
    }

    public boolean isBorrado() {
        return borrado;
    }

    public void setBorrado(boolean borrado) {
        this.borrado = borrado;
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }
    
    public void invertSelected() { selected = !selected;}

    public boolean isGroupofnames() {
        return groupofnames;
    }

    public void setGroupofnames(boolean groupofnames) {
        this.groupofnames = groupofnames;     
    }

    public boolean isNamedobject() {
        return namedobject;
    }

    public void setNamedobject(boolean namedobject) {
        this.namedobject = namedobject;
    }
    
}