package kemit.ldap.objectos;

import java.io.*;
import java.security.*;
import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;
import kemit.crypto.*;
import kemit.crypto.jcrypt;
import kemit.crypto.sambaPasswords;
import kemit.ldap.Principal;
import kemit.util.AsciiUtils;
import kemit.util.CopyFile;
import kemit.util.unixUtil;



public class UsuarioLDAP implements Serializable {
    
    private String uid = "";
    private String displayName = "";
    private byte[] userPassword;
    private String uidNumber = "";
    private String cn = "";
    private String sn = "";
    private String givenName = "";
    private String homeDirectory = "";
    private String sagaID = "";
    private String loginShell = "";
    private String gidNumber = "";
    private String employeeNumber = "";
    private String sambaLMPassword = "";
    private String sambaAcctFlags = "";
    private String sambaNTPassword = "";
    private String sambaSID = "";
    private String sambaPrimaryGroupSID = "";
    private String plainPassword="";
        
    private Vector grupos=new Vector();
    Hashtable otrosAtributos=new Hashtable();

    private boolean selected=true;
    private boolean borrado=false;
    
    private GregorianCalendar gc;
    private long segundos;
    
    static Random random=new Random();        

    public UsuarioLDAP(SearchResult sr) {
        Attributes at = sr.getAttributes();
        gc=new GregorianCalendar();
        segundos=gc.getTimeInMillis()/1000;
        
        try {    
            try {uid = (String) at.get("uid").get();}  catch (Exception e) {uid="";}
            try {displayName = (String) at.get("displayName").get();}  catch (Exception e) {displayName="";}
            try {userPassword = (byte[]) at.get("userPassword").get();} catch (NullPointerException npe) {userPassword=null;}
            try {uidNumber = (String) at.get("uidNumber").get();}  catch (Exception e) {uidNumber="";}
            try {cn = (String) at.get("cn").get();}  catch (Exception e) {cn="";}
            try {sn = (String) at.get("sn").get();}  catch (Exception e) {sn="";}
            try {givenName = (String) at.get("givenName").get();}  catch (Exception e) {givenName="";}
            try {homeDirectory = (String) at.get("homeDirectory").get();}  catch (Exception e) {homeDirectory="";}
            try {sagaID = (String) at.get("sagaID").get();}  catch (NullPointerException npe) {sagaID="";} 
            try {loginShell = (String) at.get("loginShell").get();}  catch (NullPointerException e) {loginShell="";}
            try {gidNumber = (String) at.get("gidNumber").get();}  catch (NullPointerException e) {gidNumber="";}
            try {employeeNumber = (String) at.get("employeeNumber").get();}  catch (NullPointerException e) {employeeNumber="";}
            try {sambaLMPassword = (String) at.get("sambaLMPassword").get();}  catch (NullPointerException e) {sambaLMPassword="";}
            try {sambaAcctFlags = (String) at.get("sambaAcctFlags").get();}  catch (NullPointerException e) {sambaAcctFlags="";}
            try {sambaNTPassword = (String) at.get("sambaNTPassword").get();}  catch (NullPointerException e) {sambaNTPassword="";}
            try {sambaSID = (String) at.get("sambaSID").get();}  catch (NullPointerException e) {sambaSID="";}
            try {sambaPrimaryGroupSID = (String) at.get("sambaPrimaryGroupSID").get();}  catch (NullPointerException e) {sambaPrimaryGroupSID="";} 
            //GruposLDAPdelUsuario(g);
        }  catch (Exception e)  {
            e.printStackTrace();
        }        
    }
    
    public void GruposLDAPdelUsuario(Vector htg) {      
        Vector usuarios=new Vector();
        usuarios.add(this);
        Enumeration gr= htg.elements();
        while (gr.hasMoreElements()) {
            GrupoLDAP g=(GrupoLDAP) gr.nextElement();
            if ( g.getMiembros().containsKey(getUid()) ) {
                if (!getGrupos().contains(g)) getGrupos().add(g);        
            }                        
        }
    }        

      
    public UsuarioLDAP() {

    }
    
    public UsuarioLDAP normalizaAtributos(DirContext dc,Principal padre) {
        SambaDomain sd=new SambaDomain(dc);
        
        Enumeration claves=otrosAtributos.keys();
        String n=(String)otrosAtributos.get("sn");
        if (n!=null) {setSn(n);menosAtributos("sn");}
        while (claves.hasMoreElements()) {
            String clave=(String) claves.nextElement();
            if (clave.equals("#")) {menosAtributos(clave);}
            if (clave.equals("uid")) {uid=(String) otrosAtributos.get(clave);menosAtributos(clave);}
            if (clave.equals("cn")) {cn=(String) otrosAtributos.get(clave);menosAtributos(clave);}
            if (clave.equals("givenName")) {givenName=(String) otrosAtributos.get(clave);menosAtributos(clave);}
            if (clave.equals("sagaID")) {sagaID=(String) otrosAtributos.get(clave);menosAtributos(clave);}
            if (clave.equals("sn2")) {
                String s=(String) otrosAtributos.get(clave);
                setSn(getSn()+" "+s);
                menosAtributos(clave);
            }
            if (clave.equals("grup")) {
                String s=(String) otrosAtributos.get(clave);
                if (!s.trim().equals("")) {
                    GrupoLDAP g=new GrupoLDAP();                    
                    g.setCn(s);
                    masGrupos(g);
                }
                menosAtributos(clave);
            }  
            menosAtributos(clave);
        }
        if (getDisplayName().equals("")) setDisplayName(getGivenName()+" "+getSn());
        if (getCn().equals("")) setCn(getGivenName()+" "+getSn());
        if (getUid().equals("") || getUid()==null) {
            String newuid=generaUid();            
            while (padre.isUidInNewUsers(newuid) || padre.isUidInUsers(newuid) || padre.isUidInUnix(newuid)) {
                newuid=generaUid();
            }
            setUid(newuid);
        }
        String home=padre.txtHome.getText();
        if (!home.endsWith("/")) home=home+"/";
        if (getHomeDirectory().trim().equals("")) setHomeDirectory(home+getUid());
        if (getLoginShell().equals("")) setLoginShell(padre.txtShell.getText());
        if (getUserPassword()==null) {
            String pwd=generaPwd();
            setPlainPassword(pwd);
            try {
                setSambaLMPassword(sambaPasswords.SambaLMPassword(pwd));
                setSambaNTPassword(sambaPasswords.SambaNTPassword(pwd));
            } catch (Exception e) {e.printStackTrace();}
        }             
        if (getSambaLMPassword().equals("")) {
            try {
                setSambaLMPassword(sambaPasswords.SambaLMPassword(getPlainPassword()));
                setSambaNTPassword(sambaPasswords.SambaNTPassword(getPlainPassword()));
            } catch (Exception e) {e.printStackTrace();}           
        }
        if (getUidNumber().trim().equals("")) {padre.maxUID++;setUidNumber(""+padre.maxUID);}
        if (getGidNumber().equals("")) {setGidNumber("100");}
        if (getSambaSID().equals("")) {setSambaSID(sd.getSambaSID()+"-"+getUidNumber());}
        if (getSambaPrimaryGroupSID().equals("")) {setSambaPrimaryGroupSID(sd.getSambaSID()+"-"+getGidNumber());}
        if (getSambaAcctFlags().equals("")) {setSambaAcctFlags("[U          ]");}                
        
        return this;
    }
    
    public void masAtributos(String nombre,Object o,boolean repetible) {
        if (repetible) {
             otrosAtributos.put(nombre,o);
        } else {
            if (!otrosAtributos.containsKey(nombre)) otrosAtributos.put(nombre,o);
        }
    }
    
    public void menosAtributos(String nombre) {
        otrosAtributos.remove(nombre);
    }
    
    public void masGrupos(GrupoLDAP g) {
        if (!getGrupos().contains(g)) getGrupos().add(g);        
    }
 
   public void menosGrupos(GrupoLDAP g) {
        getGrupos().remove(g);        
    }
     
    
    
    public boolean borrar(DirContext dc,Principal padre) {
        // empaquetar directorio        
        padre.mensageOp(padre.BORRANDO_USUARIO+" "+this);
        padre.setBarra(padre.getBarra()+1);
        boolean bien=true;
        padre.mensageOp("   ->"+padre.TAR_FICHERO_USUARIO);
        //no empaquetar
        if (padre.chConservarHome.isSelected())
            bien=unixUtil.ejecuta("tar -czf "+getUid()+".tgz "+getHomeDirectory(),null,padre.dirGuargar.getText());        
        padre.setBarra(padre.getBarra()+1);
        if (!bien) {
            padre.mensageOp("   ->"+padre.ERROR_TAR);
            padre.mensageOp("   ->"+"stio:"+unixUtil.getRes());
            padre.mensageOp("   ->"+"sterr:"+unixUtil.getErr());            
            return false;
        }                
        padre.mensageOp("   ->"+padre.BORRANDO_USER_EN_SERVER);
        padre.setBarra(padre.getBarra()+1);
        bien=borrarEnServerLDAP(dc,padre);
        if (!bien) {
            padre.mensageOp("   ->"+padre.ERROR_BORRANDO_USER_EN_SERVER);
            return false;
        }                                
        padre.mensageOp("   ->"+padre.BORRANDO_FICHERO_USUARIO);
        bien=unixUtil.ejecutaS("rm -R "+getHomeDirectory(),null,padre.dirGuargar.getText());        
        padre.setBarra(padre.getBarra()+1);
        if (!bien) {
            padre.mensageOp("   ->"+padre.ERROR_BORRANDO_FICHERO_USUARIO);
            padre.mensageOp("   ->"+"stio:"+unixUtil.getRes());
            padre.mensageOp("   ->"+"sterr:"+unixUtil.getErr());            
            return false;
        }                        
        return true;
    }
    
    public boolean crear(DirContext dc,Principal padre) {
        padre.mensageOp(padre.CREANDO_USUARIO+":"+this.toString());
        boolean bien=true;
        boolean copia=true;

//        Crear directorio home
        padre.setBarra(padre.getBarra()+1);
        padre.mensageOp("    ->"+padre.CREANDO_HOME);
        String home=padre.txtHome.getText();
        if (!home.endsWith("/")) home=home+"/";
        
        File hs=new File(home+getUid());
        if (!hs.exists()) {                
            bien=unixUtil.ejecuta("mkdir "+home+getUid()+"/",null,home);
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_CREANDO_HOME);
                padre.mensageOp("stio:"+unixUtil.getRes());
                padre.mensageOp("sterr:"+unixUtil.getErr());
                return false;
            }
            copia=true;
        } else copia=false;
        
//        Copiar el skel

        padre.setBarra(padre.getBarra()+1);
        padre.mensageOp("    ->"+padre.COPIANDO_SKEL);
//      copia skel si no existe nada en el directorio

        if (copia) {
            try {
                bien=CopyFile.copia("/etc/skel",home+getUid());
            } catch (IOException ex) {
                if (padre.debug) padre.mensage(ex);
                //bien=false;
            }
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_COPIANDO_SKEL);
                return false;
            }
        }
        padre.setBarra(padre.getBarra()+1);
        bien=crearEnServerLDAP(dc,padre);
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_GRABACION_EN_SERVER);
            return false;
        }                
        
        padre.setBarra(padre.getBarra()+2);
        padre.mensageOp("    ->"+padre.ASIGNANDO_DERECHOS);
        
        bien=unixUtil.ejecutaS("chown -R "+getUid()+" "+home+getUid()+"/",null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                
        bien=unixUtil.ejecutaS("chgrp -R "+getGidNumber()+" "+home+getUid()+"/",null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                        
        return true;
    }
    
    
    public boolean crearManual(DirContext dc,Principal padre) {
        padre.mensageOp(padre.CREANDO_USUARIO+":"+this.toString());
        boolean bien=true;
        boolean copia=true;
//        Crear directorio home
        padre.mensageOp("    ->"+padre.CREANDO_HOME);
        String home=padre.txtHomeUsu.getText();
        if (!home.endsWith("/")) home=home+"/";
        
        File hs=new File(home);
        if (!hs.exists()) {                
            bien=unixUtil.ejecuta("mkdir "+home,null,null);
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_CREANDO_HOME);
                padre.mensageOp("stio:"+unixUtil.getRes());
                padre.mensageOp("sterr:"+unixUtil.getErr());
                return false;
            }
            copia=true;
        } else copia=false;
        
//        Copiar el skel
        padre.mensageOp("    ->"+padre.COPIANDO_SKEL);
//      copia skel si no existe nada en el directorio

        if (copia) {
            try {
                bien=CopyFile.copia("/etc/skel",home);
            } catch (IOException ex) {
                if (padre.debug) padre.mensage(ex);
                //bien=false;
            }
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_COPIANDO_SKEL);
                return false;
            }
        }
        bien=crearEnServerLDAP(dc,padre);
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_GRABACION_EN_SERVER);
            return false;
        }                        
        padre.mensageOp("    ->"+padre.ASIGNANDO_DERECHOS);        
        bien=unixUtil.ejecutaS("chown -R "+getUid()+" "+home,null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                
        bien=unixUtil.ejecutaS("chgrp -R "+getGidNumber()+" "+home,null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                        
        return true;
    }
    
    public boolean modificarManual(DirContext dc,Principal padre) {
        padre.mensageOp(padre.CREANDO_USUARIO+":"+this.toString());
        boolean bien=true;
        boolean copia=true;
//        Crear directorio home
        padre.mensageOp("    ->"+padre.CREANDO_HOME);
        String home=padre.txtHomeUsu.getText();
        if (!home.endsWith("/")) home=home+"/";
        
        File hs=new File(home);
        if (!hs.exists()) {                
            bien=unixUtil.ejecuta("mkdir "+home,null,null);
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_CREANDO_HOME);
                padre.mensageOp("stio:"+unixUtil.getRes());
                padre.mensageOp("sterr:"+unixUtil.getErr());
                return false;
            }
            copia=true;
        } else copia=false;
        
        if (copia) {
            try {
                padre.mensageOp("    ->"+padre.COPIANDO_SKEL);
                bien=CopyFile.copia("/etc/skel",home);
            } catch (IOException ex) {
                if (padre.debug) padre.mensage(ex);
                //bien=false;
            }
            if (!bien) {
                padre.mensageOp("    ->"+padre.ERROR_COPIANDO_SKEL);
                return false;
            }
        }
        padre.mensageOp("    ->"+padre.ASIGNANDO_DERECHOS);        
        bien=unixUtil.ejecutaS("chown -R "+getUid()+" "+home,null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                
        bien=unixUtil.ejecutaS("chgrp -R "+getGidNumber()+" "+home,null,home);        
        if (!bien) {
            padre.mensageOp("    ->"+padre.ERROR_ASIGNANDO_DERECHOS);
            padre.mensageOp("stio:"+unixUtil.getRes());
            padre.mensageOp("sterr:"+unixUtil.getErr());            
            return false;
        }                        
        return true;
    }
    
    
    public boolean borrarEnServerLDAP(DirContext dc,Principal padre) {        
        
        try {     
            Enumeration e=getGrupos().elements();
            while (e.hasMoreElements()) {
                GrupoLDAP g=(GrupoLDAP) e.nextElement();
                g.minusMemberLDAP(dc,this,padre.txtDC.getText());
            }            
            dc.destroySubcontext("uid="+getUid()+", ou=people");
            return true;
        } catch (Exception ex) {
            if (padre.debug) padre.mensage(ex);
            return false;
        }
    }
       
    public String generaUid() {                
        String tuid="";        
        if (getGivenName().equals("")) tuid+="x"; else {
            StringTokenizer st=new StringTokenizer(getGivenName());
            while (st.hasMoreTokens()) {
                String s=st.nextToken();
                tuid+=s.substring(0,1);
            }            
        }
        StringTokenizer st=new StringTokenizer(getSn());
        while (st.hasMoreTokens()) {
            String s=st.nextToken();
            if (s.length()>3) tuid+=s.substring(0,3); else tuid+=s;
        }
        
        tuid=tuid+random.nextInt(90);          
        return AsciiUtils.convertNonAscii(tuid.toLowerCase());
    }
    
    public String generaPwd() {        
        RandPass rp=new RandPass();
        return rp.getPass(8);
    }
    
    
    public boolean modificarEnServerLDAP(DirContext dc,Principal padre) {
       try {              
           String algoritmo="LINUX";
            switch (padre.getAlgoritmo()) {
                case 0:     algoritmo="PLAIN";break;
                case 1:     algoritmo="LINUX";break;
                case 2:     algoritmo="SHA";break;
                case 3:     algoritmo="MD5";break;
                case 4:     algoritmo="SSHA";break;
                case 5:     algoritmo="SMD5";break;                    
                default:    algoritmo="LINUX";
            }                      
          
            String pwd=getPlainPassword();

            Vector modificaciones=new Vector();
            ModificationItem m;
            Attribute att;    
            
            if (!pwd.equals("")) {
                if (padre.getSAMBA()==1) {
                    att = new BasicAttribute("sambaLMPassword",sambaPasswords.SambaLMPassword(pwd));                
                    m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
                    modificaciones.add(m);
                    att = new BasicAttribute("sambaNTPassword",sambaPasswords.SambaNTPassword(pwd));
                    m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
                    modificaciones.add(m);
                    gc=new GregorianCalendar();
                    segundos=gc.getTimeInMillis()/1000;                    
                    att = new BasicAttribute("sambaPWDLastSet",""+segundos);
                    m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
                    modificaciones.add(m);                                        
                }
                att = new BasicAttribute("userPassword",encripta(pwd,algoritmo));
                m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
                modificaciones.add(m);
            }
            att = new BasicAttribute("homeDirectory",getHomeDirectory());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);

            att = new BasicAttribute("loginShell",getLoginShell());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);

            att = new BasicAttribute("sn",getSn());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);
            
            att = new BasicAttribute("givenName",getGivenName());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);

            att = new BasicAttribute("cn",getCn());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);
            
            att = new BasicAttribute("sagaID",getSagaID());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);

            att = new BasicAttribute("gidNumber",getGidNumber());
            m = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,att);
            modificaciones.add(m);
            
            
            ModificationItem[] mods = new ModificationItem[modificaciones.size()];
            for (int i=0;i<modificaciones.size();i++) {
                mods[i]=(ModificationItem)modificaciones.elementAt(i);
            }
            
            
            dc.modifyAttributes("uid="+getUid()+",ou=people", mods);              
            //dc.close();
                    
            return true;
            //JOptionPane.showMessageDialog(this,OK_CAMBIO,ATT,JOptionPane.INFORMATION_MESSAGE);            
        } catch (Exception ex) {
            if (padre.debug) ex.printStackTrace();            
        }       
        
        
        //ponGruposUsuario(u);
       
       return false;
        
    }
    
    
        
    public boolean crearEnServerLDAP(DirContext dc,Principal padre) {        

        padre.mensageOp("    ---->"+padre.NORMALIZA_ATT);
        normalizaAtributos(dc,padre);
        padre.setBarra(padre.getBarra()+1);
        try {          
            padre.mensageOp("    ---->"+padre.PONE_ATT);
            String distinguishedName = "uid=" + getUid() + ",ou=people" ;           
            Attributes newAttributes = new BasicAttributes(true);            
            Attribute oc = new BasicAttribute("objectclass");
            
            oc.add("posixAccount");
            oc.add("inetOrgPerson");
            oc.add("shadowAccount");
            oc.add("top");
            if (padre.getSAMBA()==1) oc.add("sambaSamAccount");
            newAttributes.put(oc);                        
            
            String algoritmo="LINUX";
            switch (padre.getAlgoritmo()) {
                case 0:     algoritmo="PLAIN";break;
                case 1:     algoritmo="LINUX";break;
                case 2:     algoritmo="SHA";break;
                case 3:     algoritmo="MD5";break;
                case 4:     algoritmo="SSHA";break;
                case 5:     algoritmo="SMD5";break;
                    
                default:    algoritmo="LINUX";
            }
            
            padre.setBarra(padre.getBarra()+1);
            newAttributes.put(new BasicAttribute("userPassword", encripta(getPlainPassword(),algoritmo) ));            
            newAttributes.put(new BasicAttribute("cn", getCn()));
            newAttributes.put(new BasicAttribute("givenName",getGivenName()));
            newAttributes.put(new BasicAttribute("sn",getSn()));
            newAttributes.put(new BasicAttribute("uid", getUid()));
            newAttributes.put(new BasicAttribute("sagaID", getSagaID()));
            newAttributes.put(new BasicAttribute("displayName", getDisplayName()));
                       
            newAttributes.put(new BasicAttribute("gidNumber", getGidNumber()));            
            newAttributes.put(new BasicAttribute("uidNumber", getUidNumber()));
            
            String home=padre.txtHome.getText();            
            if (getHomeDirectory().trim().equals("")) setHomeDirectory(home+getUid());
            if (!home.endsWith("/")) home=home+"/";
            
            newAttributes.put(new BasicAttribute("homeDirectory", getHomeDirectory()));
            newAttributes.put(new BasicAttribute("loginShell", getLoginShell()));
            
            if (padre.getSAMBA()==1) {
                newAttributes.put(new BasicAttribute("sambaSID", getSambaSID() ));
                newAttributes.put(new BasicAttribute("sambaPrimaryGroupSID", getSambaPrimaryGroupSID()));
                newAttributes.put(new BasicAttribute("sambaAcctFlags", getSambaAcctFlags()));
                newAttributes.put(new BasicAttribute("sambaLMPassword", getSambaLMPassword() ));
                newAttributes.put(new BasicAttribute("sambaNTPassword", getSambaNTPassword() ));
                gc=new GregorianCalendar();
                segundos=gc.getTimeInMillis()/1000;
                newAttributes.put(new BasicAttribute("sambaPWDLastSet", ""+segundos ));
            }
            
            padre.setBarra(padre.getBarra()+1);
            Enumeration claves=otrosAtributos.keys();
            while (claves.hasMoreElements()) {
                String clave=(String)claves.nextElement();                
                newAttributes.put(new BasicAttribute(clave, otrosAtributos.get(clave)));                
            }            
            padre.mensageOp("    ---->"+padre.ENVIA_AL_SERVER);
            dc.createSubcontext(distinguishedName, newAttributes);
            padre.setBarra(padre.getBarra()+2);
            Enumeration grups=getGrupos().elements();
            while (grups.hasMoreElements()) {
                GrupoLDAP g=(GrupoLDAP) grups.nextElement();
                try { g.addMemberLDAP(dc,this,padre.txtDC.getText()); } 
                catch (NameNotFoundException noesta) {
                    g.newLDAP(dc,padre);
                    g.addMemberLDAP(dc,this,padre.txtDC.getText());
                }
            }            
            padre.mensageOp("    ---->"+padre.CREADO_EN_SERVER);
            return true;
        }
        catch (Exception e) {
            if (padre.debug) padre.mensage(e);
            return false;
        }        
        
    }
 
    public boolean newLDAP(DirContext dc,Principal padre) {        

        //padre.setBarra(padre.getBarra()+1);
        normalizaAtributos(dc,padre);
        try {          
            padre.mensageOp("    ---->"+padre.PONE_ATT);
            String distinguishedName = "uid=" + getUid() + ",ou=people" ;           
            Attributes newAttributes = new BasicAttributes(true);            
            Attribute oc = new BasicAttribute("objectclass");
            
            oc.add("posixAccount");
            oc.add("inetOrgPerson");
            oc.add("shadowAccount");
            oc.add("top");
            if (padre.getSAMBA()==1) oc.add("sambaSamAccount");
            newAttributes.put(oc);                        
                                    
            //padre.setBarra(padre.getBarra()+1);
            newAttributes.put(new BasicAttribute("userPassword", new String(getUserPassword()) ));            
            newAttributes.put(new BasicAttribute("cn", getCn()));
            newAttributes.put(new BasicAttribute("givenName",getGivenName()));
            newAttributes.put(new BasicAttribute("sn",getSn()));
            newAttributes.put(new BasicAttribute("uid", getUid()));
            newAttributes.put(new BasicAttribute("sagaID", getSagaID()));
            newAttributes.put(new BasicAttribute("displayName", getDisplayName()));
                       
            newAttributes.put(new BasicAttribute("gidNumber", getGidNumber()));            
            newAttributes.put(new BasicAttribute("uidNumber", getUidNumber()));                       
            
            newAttributes.put(new BasicAttribute("homeDirectory", getHomeDirectory()));
            newAttributes.put(new BasicAttribute("loginShell", getLoginShell()));
            
            if (padre.getSAMBA()==1) {
                newAttributes.put(new BasicAttribute("sambaSID", getSambaSID() ));
                newAttributes.put(new BasicAttribute("sambaPrimaryGroupSID", getSambaPrimaryGroupSID()));
                newAttributes.put(new BasicAttribute("sambaAcctFlags", getSambaAcctFlags()));
                newAttributes.put(new BasicAttribute("sambaLMPassword", getSambaLMPassword() ));
                newAttributes.put(new BasicAttribute("sambaNTPassword", getSambaNTPassword() ));
                gc=new GregorianCalendar();
                segundos=gc.getTimeInMillis()/1000;               
                newAttributes.put(new BasicAttribute("sambaPWDLastSet", ""+segundos ));                
            }
            //padre.setBarra(padre.getBarra()+1);
            Enumeration claves=otrosAtributos.keys();
            while (claves.hasMoreElements()) {
                String clave=(String)claves.nextElement();                
                newAttributes.put(new BasicAttribute(clave, otrosAtributos.get(clave)));                
            }            
            padre.mensageOp("    ---->"+padre.ENVIA_AL_SERVER);
            dc.createSubcontext(distinguishedName, newAttributes);
            //padre.setBarra(padre.getBarra()+2);
            Enumeration grups=getGrupos().elements();
            while (grups.hasMoreElements()) {
                GrupoLDAP g=(GrupoLDAP) grups.nextElement();
                g.addMemberLDAP(dc,this,padre.txtDC.getText());
            }            
            padre.mensageOp("    ---->"+padre.CREADO_EN_SERVER);
            return true;
        }
        catch (Exception e) {
            if (padre.debug) padre.mensage(e);
            return false;
        }        
        
    }
    
        
    
    public String getUid() {     
        return uid;
    }

    public void setUid(String uid) {
        this.uid = uid;
    }

    public String getDisplayName() {
        return displayName;
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    public byte[] getUserPassword() {
        return userPassword;
    }

    public void setUserPassword(byte[] userPassword) {
        this.userPassword = userPassword;
    }

    public String getUidNumber() {
        return uidNumber;
    }

    public void setUidNumber(String uidNumber) {
        this.uidNumber = uidNumber;
    }

    public String getCn() {
        return cn;
    }

    public void setCn(String cn) {
        this.cn = cn;
    }

    public String getHomeDirectory() {       
        return homeDirectory;
    }

    public void setHomeDirectory(String homeDirectory) {
        this.homeDirectory = homeDirectory;
    }

    public String getSagaID() {
        return sagaID;
    }

    public void setSagaID(String sagaID) {
        this.sagaID = sagaID;
    }

    public String getLoginShell() {
        return loginShell;
    }

    public void setLoginShell(String loginShell) {
        this.loginShell = loginShell;
    }

    public String getGidNumber() {
        return gidNumber;
    }

    public void setGidNumber(String gidNumber) {
        this.gidNumber = gidNumber;
    }

    public String getEmployeeNumber() {
        return employeeNumber;
    }

    public void setEmployeeNumber(String employeeNumber) {
        this.employeeNumber = employeeNumber;
    }

    public String getSambaLMPassword() {
        return sambaLMPassword;
    }

    public void setSambaLMPassword(String sambaLMPassword) {
        this.sambaLMPassword = sambaLMPassword;
    }

    public String getSambaAcctFlags() {
        return sambaAcctFlags;
    }

    public void setSambaAcctFlags(String sambaAcctFlags) {
        this.sambaAcctFlags = sambaAcctFlags;
    }

    public String getSambaNTPassword() {
        return sambaNTPassword;
    }

    public void setSambaNTPassword(String sambaNTPassword) {
        this.sambaNTPassword = sambaNTPassword;
    }

    public String getSambaSID() {
        return sambaSID;
    }

    public void setSambaSID(String sambaSID) {
        this.sambaSID = sambaSID;
    }

    public String getSambaPrimaryGroupSID() {
        return sambaPrimaryGroupSID;
    }

    public void setSambaPrimaryGroupSID(String sambaPrimaryGroupSID) {
        this.sambaPrimaryGroupSID = sambaPrimaryGroupSID;
    }

    public String getSn() {
        return sn;
    }

    public void setSn(String sn) {
        this.sn = sn;
    }

    public String getPlainPassword() {
        return plainPassword;
    }

    public void setPlainPassword(String plainPassword) {
        this.plainPassword = plainPassword;        
        setUserPassword(plainPassword);
    }
    public void setUserPassword(String s) {
        try {
            this.userPassword=s.getBytes("UTF-8");
        } catch (UnsupportedEncodingException ex) {
            ex.printStackTrace();
        }
    }
    
    public String encripta(String s,String modo){
                        
        if (modo.toUpperCase().equals("PLAIN")) return s;
        else if (modo.toUpperCase().equals("LINUX")) return "{crypt}"+jcrypt.crypt(s);
        else if (modo.toUpperCase().equals("SHA")) return new String(crypUtil.mdDecode(s, 4, null));
        else if (modo.toUpperCase().equals("MD5")) return new String(crypUtil.mdDecode(s, 2, null));
        else if (modo.toUpperCase().equals("SSHA")) return new String(crypUtil.mdDecode(s, 5, saltando()));
        else if (modo.toUpperCase().equals("SMD5")) return new String(crypUtil.mdDecode(s, 3, saltando()));
        else return s;
    }
    
    public byte[] saltando() {
        
        final String SALTCHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
        
        StringBuffer salt = new StringBuffer();
        java.util.Random rnd = new java.util.Random();
        
        while (salt.length() < 8) {
            int index = (int) (rnd.nextFloat() * SALTCHARS.length());
            salt.append(SALTCHARS.substring(index, index+1));
        }
        return salt.toString().getBytes();
    }
 
    public String toStringC() {
        return "cn="+getCn()+" uid="+getUid()+" sagaID="+getSagaID()+" uidNumber="+getUidNumber();
    }
    
    
    public String toString() {
        return "cn="+getCn()+",uid="+getUid();
    }

    public String getGivenName() {
        return givenName;
    }

    public void setGivenName(String givenName) {
        this.givenName = givenName;
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }
    
    public void invertSelected() { selected = !selected;}
    
    
    public boolean equals(Object o) {
        if (o==null) return false;
        if (o instanceof UsuarioLDAP) {
            UsuarioLDAP u=(UsuarioLDAP)o;
            if (getSagaID().equals("")) return getUid().equals(u.getUid());
            else return getSagaID().equals(u.getSagaID()) ;            
        } else return false;
    }

    public boolean isBorrado() {
        return borrado;
    }

    public void setBorrado(boolean borrado) {
        this.borrado = borrado;
    }

    public Vector getGrupos() {
        return grupos;
    }

    public void setGrupos(Vector grupos) {
        this.grupos = grupos;
    }
    
    
}