package kemit.ssl;

import javax.net.ssl.*;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.awt.*;

public class KemiTrustManager implements X509TrustManager {
    
    public static String CERTSTORE = "lbecacerts";
    public static String CERTSTORE_PASSWORD = "changeit";
    private X509TrustManager trustManager;

    private KeyStore caKeyStore;
    private String caKeyStorePath;
    private String caKeyStoreType;
    private char[] caPassphrase;

    public static X509TrustManager[] convert(TrustManager[] arrayOfTrustManagers, 
                                            KeyStore keystore,
                                            String caKeystorePath, 
                                            char[] caPassphrase, 
                                            String caKeystoreType) throws SSLException   {
        int numberOfManagers = arrayOfTrustManagers.length;
        X509TrustManager[] returnArray = new X509TrustManager[numberOfManagers];
        for (int i = 0; i < numberOfManagers; i++) {
            TrustManager old = arrayOfTrustManagers[i];
            if (old == null) throw new SSLException("unexpected SSL error - null trust manager found in trust array: element " + i + " of " + numberOfManagers);
            try {
                returnArray[i] = new KemiTrustManager((X509TrustManager) old, keystore, caKeystorePath, caPassphrase, caKeystoreType);
            }
            catch (ClassCastException e) {
                throw new SSLException("unexpected SSL error - non X509 trust manager found in trust array: element " + i + " of " + numberOfManagers + " is of type " + old.getClass());
            }
        }
        return returnArray;
    }

    private KemiTrustManager(X509TrustManager baseTrustManager, KeyStore keystore,String certAuthorityKeystorePath, char[] certAuthorityPassphrase, String certAuthorityKeystoreType) {
        trustManager = baseTrustManager;
        caKeyStore = keystore;
        caKeyStorePath = certAuthorityKeystorePath;
        caKeyStoreType = certAuthorityKeystoreType;
        caPassphrase = certAuthorityPassphrase;
    }

    public X509Certificate[] getAcceptedIssuers() {
        return trustManager.getAcceptedIssuers();
    }

    private X509Certificate getCACert(X509Certificate chain[]) {
        X509Certificate ca = chain[chain.length - 1];
        
        if (ca.getSubjectDN().equals(ca.getIssuerDN()))
            return ca;
        else
            return null;
    }

    private boolean rootCertIsKnown(X509Certificate caCert) {
        X509Certificate certificates[] = getAcceptedIssuers();
        if (certificates == null) return false;
        for (int i = 0; i < certificates.length; i++) if (caCert.equals(certificates[i])) return true;
        return false;
    }

    public void checkClientTrusted(X509Certificate chain[], String authType) throws CertificateException {
        trustManager.checkClientTrusted(chain, authType);
    }


    public void checkServerTrusted(X509Certificate chain[], String authType) throws CertificateException {
        try {
            trustManager.checkServerTrusted(chain, authType);  
        }
        catch (CertificateException e) {
            if ("false".equals(System.getProperty("option.ssl.import.cert.during.connection"))) throw e;

            X509Certificate certificateAuthorityCert = getCACert(chain);
            if (certificateAuthorityCert == null)
                throw new CertificateException("Invalid Server Certificate: server certificate could not be verified, and the CA certificate is missing from the certificate chain. raw error: " + e);

            if (rootCertIsKnown(certificateAuthorityCert))
                throw new CertificateException("Invalid Server Certificate: The server certificate could not be verified, as it has a bad chain back to a known CA.  raw error: " + e);

        }
    }

    private void saveStore(X509Certificate cert) throws KeyStoreException {
        try {
            if (caPassphrase == null)  caPassphrase = "changeit".toCharArray();  
            try {
                caKeyStore.load(new FileInputStream(caKeyStorePath), caPassphrase); // if this works, we have a valid password.
            } 
            catch (IOException e) {setupKeyStoreAndPassword();}

            if (caKeyStore == null)
                throw new KeyStoreException("unable to open keystore - no valid password or no valid file.");

            String alias = cert.getSubjectDN() + " (" + cert.getSerialNumber().toString() + ")";
            caKeyStore.setCertificateEntry(alias, cert);
            FileOutputStream fos = new FileOutputStream(caKeyStorePath);
            caKeyStore.store(fos, caPassphrase);
            fos.close();
        }
        catch (IOException e) {
            KeyStoreException kse = new KeyStoreException("unable to access keystore file");
            kse.initCause(e);
            throw kse;
        }
        catch (GeneralSecurityException e) {
            if (e instanceof KeyStoreException) throw (KeyStoreException)e;
            else throw new KeyStoreException("unable to save keystore " + caKeyStorePath + " error was: " + e);
        }
    }
    
    private boolean setupKeyStoreAndPassword()
    {
        return true;
//        String message = CBIntText.get("Enter Key Store Password");
//        while ((caPassphrase = KeystoreGUI.getPassword(owner, message)) != null)
//        {
//            caKeyStore = KeystoreGUI.readKeyStore(caPassphrase, caKeyStoreType, caKeyStorePath);
//
//            if (caKeyStore != null)
//                return true;    // we have a valid keystore!
//
//            // this message is only displayed if we go around the loop again.
//            message = CBIntText.get("Password incorrect. Please try again.");
//        }
//        return false;
    }

}