
package kemit.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JDialog;

public class SplashWindow extends Window {
    private Image splashImage;
    private JDialog dlg;

    private boolean paintCalled = false;

    public SplashWindow(Frame owner, JDialog dlg) {
        
        super(owner);
        this.dlg=dlg;                
        
        int imgWidth = dlg.getContentPane().getWidth();
        int imgHeight = dlg.getContentPane().getHeight();
        setSize(imgWidth, imgHeight);
        
        add(dlg.getContentPane());        
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(
            (screenDim.width - imgWidth) / 2,
            (screenDim.height - imgHeight) / 2
        );                        
    }
    
    
    public SplashWindow(Frame owner, Image splashImage) {
        super(owner);
        this.splashImage = splashImage;

        MediaTracker mt = new MediaTracker(this);
        mt.addImage(splashImage,0);
        try { mt.waitForID(0);} catch(InterruptedException ie){}

        int imgWidth = splashImage.getWidth(this);
        int imgHeight = splashImage.getHeight(this);
        setSize(imgWidth, imgHeight);
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(
            (screenDim.width - imgWidth) / 2,
            (screenDim.height - imgHeight) / 2
        );

        MouseAdapter disposeOnClick = new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                synchronized(SplashWindow.this) {
                    SplashWindow.this.paintCalled = true;
                    SplashWindow.this.notifyAll();
                }
                dispose();
            }
        };
        addMouseListener(disposeOnClick);
    }

    public void update(Graphics g) {
        g.setColor(getForeground());
        paint(g);
    }

    public void paint(Graphics g) {
        g.drawImage(splashImage, 0, 0, this);
    if (! paintCalled) {
            paintCalled = true;
            synchronized (this) { notifyAll(); }
        }
    }

    public static Frame splash(Image splashImage) {
        Frame f = new Frame();
        SplashWindow w = new SplashWindow(f, splashImage);

        w.toFront();
        w.setVisible(true);

        if (! EventQueue.isDispatchThread()) {
            synchronized (w) {
                while (! w.paintCalled) {
                    try { w.wait(); } catch (InterruptedException e) {}
                }
            }
        }
        return f;
    }
    
    public static Frame splash(JDialog dlg) {
        Frame f = new Frame();
        SplashWindow w = new SplashWindow(f, dlg);

        w.toFront();
        w.setVisible(true);

        if (! EventQueue.isDispatchThread()) {
            synchronized (w) {
                while (! w.paintCalled) {
                    try { w.wait(); } catch (InterruptedException e) {}
                }
            }
        }
        return f;
    }
    
    
}
